@@ -33,6 +33,7 @@ G_DEFINE_TYPE (GVfsAfpVolume, g_vfs_afp_volume, G_TYPE_OBJECT);
 struct _GVfsAfpVolumePrivate
 {
   GVfsAfpServer *server;
+  GVfsAfpConnection *conn;
   gboolean mounted;
 
   guint16 attributes;
@@ -68,17 +69,19 @@ g_vfs_afp_volume_class_init (GVfsAfpVolumeClass *klass)
 }
 
 GVfsAfpVolume *
-g_vfs_afp_volume_new (GVfsAfpServer *server)
+g_vfs_afp_volume_new (GVfsAfpServer *server, GVfsAfpConnection *conn)
 {
   GVfsAfpVolume *volume;
   GVfsAfpVolumePrivate *priv;
 
   g_return_val_if_fail (G_VFS_IS_AFP_SERVER (server), NULL);
+  g_return_val_if_fail (G_VFS_IS_AFP_CONNECTION (conn), NULL);
   
   volume = g_object_new (G_VFS_TYPE_AFP_VOLUME, NULL);
   priv = volume->priv;
 
   priv->server = server;
+  priv->conn = conn;
 
   return volume;
 }
@@ -92,7 +95,6 @@ g_vfs_afp_volume_mount_sync (GVfsAfpVolume *volume,
   GVfsAfpVolumePrivate *priv;
   GVfsAfpCommand *comm;
   GVfsAfpReply *reply;
-  gboolean res;
   AfpResultCode res_code;
 
   g_return_val_if_fail (G_VFS_IS_AFP_VOLUME (volume), FALSE);
@@ -112,13 +114,9 @@ g_vfs_afp_volume_mount_sync (GVfsAfpVolume *volume,
 
   /* TODO: password? */
 
-  res = g_vfs_afp_connection_send_command_sync (priv->server->conn, comm, cancellable,
-                                                error);
+  reply = g_vfs_afp_connection_send_command_sync (priv->conn, comm, cancellable,
+                                                  error);
   g_object_unref (comm);
-  if (!res)
-    return FALSE;
-
-  reply = g_vfs_afp_connection_read_reply_sync (priv->server->conn, cancellable, error);
   if (!reply)
     return FALSE;
 
@@ -145,7 +143,7 @@ generic_error:
   /* Translators: first %s is volumename and second servername */ 
   g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                _("Couldn't mount AFP volume %s on %s"), volume_name,
-               priv->server->server_name);
+               g_vfs_afp_server_get_info(priv->server)->server_name);
   return FALSE;
 }
 
@@ -302,7 +300,7 @@ g_vfs_afp_volume_get_parms (GVfsAfpVolume       *volume,
                                       g_vfs_afp_volume_get_parms);
                                       
   
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL, get_vol_parms_cb,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL, get_vol_parms_cb,
                                      cancellable, simple);
   g_object_unref (comm);
 }
@@ -355,7 +353,7 @@ open_fork_data_free (OpenForkData *data)
 }
 
 static void
-open_fork_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
+open_fork_cb (GObject *source_object, GAsyncResult *result, gpointer user_data)
 {
   GVfsAfpConnection *conn = G_VFS_AFP_CONNECTION (source_object);
   GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (user_data);
@@ -365,6 +363,7 @@ open_fork_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
   GVfsAfpReply *reply;
   GError *err = NULL;
   AfpResultCode res_code;
+  gboolean res;
 
   OpenForkData *data;
   guint16 file_bitmap;
@@ -372,7 +371,7 @@ open_fork_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
   volume = G_VFS_AFP_VOLUME (g_async_result_get_source_object (G_ASYNC_RESULT (simple)));
   priv = volume->priv;
   
-  reply = g_vfs_afp_connection_send_command_finish (conn, res, &err);
+  reply = g_vfs_afp_connection_send_command_finish (conn, result, &err);
   if (!reply)
   {
     g_simple_async_result_take_error (simple, err);
@@ -415,8 +414,13 @@ open_fork_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
   g_vfs_afp_reply_read_int16  (reply, &data->fork_refnum);
 
   data->info = g_file_info_new ();
-  g_vfs_afp_server_fill_info (priv->server, data->info, reply, FALSE, file_bitmap);
+  res = g_vfs_afp_server_fill_info (priv->server, data->info, reply, FALSE, file_bitmap, &err);
   g_object_unref (reply);
+  if (!res)
+  {
+    g_simple_async_result_take_error (simple, err);
+    goto done;
+  }
 
   g_simple_async_result_set_op_res_gpointer (simple, data,
                                              (GDestroyNotify)open_fork_data_free);
@@ -486,7 +490,7 @@ g_vfs_afp_volume_open_fork (GVfsAfpVolume      *volume,
   simple = g_simple_async_result_new (G_OBJECT (volume), callback,
                                       user_data, g_vfs_afp_volume_open_fork);
   
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL,
                                      open_fork_cb, cancellable, simple);
   g_object_unref (comm);
 }
@@ -598,7 +602,7 @@ g_vfs_afp_volume_close_fork (GVfsAfpVolume       *volume,
   simple = g_simple_async_result_new (G_OBJECT (volume), callback, user_data,
                                       g_vfs_afp_volume_close_fork);
   
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL,
                                      close_fork_cb, cancellable, simple);
   g_object_unref (comm);
 }
@@ -734,7 +738,7 @@ g_vfs_afp_volume_delete (GVfsAfpVolume       *volume,
   simple = g_simple_async_result_new (G_OBJECT (volume), callback,
                                       user_data, g_vfs_afp_volume_delete);
   
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL,
                                      delete_cb, cancellable, simple);
   g_object_unref (comm);
 }
@@ -888,7 +892,7 @@ create_file_get_filedir_parms_cb (GObject *source_object, GAsyncResult *res, gpo
   g_vfs_afp_command_put_pathname (comm, basename);
   g_free (basename);
 
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL,
                                      create_file_cb, cfd->cancellable, simple);
   g_object_unref (comm);
 }
@@ -1076,7 +1080,7 @@ create_directory_get_filedir_parms_cb (GObject *source_object, GAsyncResult *res
   /* Pathname */
   g_vfs_afp_command_put_pathname (comm, cdd->basename);
   
-  g_vfs_afp_connection_send_command (volume->priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (volume->priv->conn, comm, NULL,
                                      make_directory_cb, cdd->cancellable, simple);
   g_object_unref (comm);
   return;
@@ -1286,7 +1290,7 @@ rename_get_filedir_parms_cb (GObject      *source_object,
   /* NewName */
   g_vfs_afp_command_put_pathname (comm, rd->new_name);
 
-  g_vfs_afp_connection_send_command (volume->priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (volume->priv->conn, comm, NULL,
                                      rename_cb, rd->cancellable, simple);
   g_object_unref (comm);
 }
@@ -1485,7 +1489,7 @@ g_vfs_afp_volume_move_and_rename (GVfsAfpVolume      *volume,
   simple = g_simple_async_result_new (G_OBJECT (volume), callback,
                                       user_data, g_vfs_afp_volume_move_and_rename);
   
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL,
                                      move_and_rename_cb, cancellable, simple);
   g_object_unref (comm);
 }
@@ -1645,7 +1649,7 @@ g_vfs_afp_volume_copy_file (GVfsAfpVolume      *volume,
   simple = g_simple_async_result_new (G_OBJECT (volume), callback,
                                       user_data, g_vfs_afp_volume_copy_file);
 
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL,
                                      copy_file_cb, cancellable, simple);
   g_object_unref (comm);
 }
@@ -1681,183 +1685,8 @@ g_vfs_afp_volume_copy_file_finish (GVfsAfpVolume *volume,
   return TRUE;
 }
 
-typedef struct
-{
-  AfpMapIDFunction function;
-  char *name;
-} MapIDData;
-
 static void
-map_id_data_free (MapIDData *mid)
-{
-  g_free (mid->name);
-
-  g_slice_free (MapIDData, mid);
-}
-
-static void
-map_id_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  GVfsAfpConnection *conn = G_VFS_AFP_CONNECTION (source_object);
-  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (user_data);
-
-  GVfsAfpReply *reply;
-  GError *err = NULL;
-
-  AfpResultCode res_code;
-  MapIDData *map_data;
-
-  reply = g_vfs_afp_connection_send_command_finish (conn, res, &err);
-  if (!reply)
-  {
-    g_simple_async_result_take_error (simple, err);
-    g_simple_async_result_complete (simple);
-    return;
-  }
-
-  res_code = g_vfs_afp_reply_get_result_code (reply);
-  if (res_code != AFP_RESULT_NO_ERROR)
-  {
-    switch (res_code)
-    {
-      case AFP_RESULT_ITEM_NOT_FOUND:
-        g_simple_async_result_set_error (simple, G_IO_ERROR, G_IO_ERROR_FAILED,
-                                         _("ID not found"));
-        break;
-      default:
-        g_simple_async_result_take_error (simple, afp_result_code_to_gerror (res_code));
-        break;
-    }
-
-    g_simple_async_result_complete (simple);
-    return;
-  }
-
-  map_data = g_simple_async_result_get_op_res_gpointer (simple);
-  
-  if (map_data->function == AFP_MAP_ID_FUNCTION_USER_UUID_TO_UTF8_NAME ||
-      map_data->function == AFP_MAP_ID_FUNCTION_GROUP_UUID_TO_UTF8_NAME)
-  {
-    /* objType */
-    g_vfs_afp_reply_read_uint32 (reply, NULL);
-    /* id */
-    g_vfs_afp_reply_read_uint32 (reply, NULL);
-  }
-
-  if (map_data->function == AFP_MAP_ID_FUNCTION_USER_ID_TO_NAME ||
-      map_data->function == AFP_MAP_ID_FUNCTION_GROUP_ID_TO_NAME)
-  {
-    g_vfs_afp_reply_read_pascal (reply, &map_data->name);
-  }
-  else
-  {
-    GVfsAfpName *afp_name;
-
-    g_vfs_afp_reply_read_afp_name (reply, FALSE, &afp_name);
-    map_data->name = g_vfs_afp_name_get_string (afp_name);
-    g_vfs_afp_name_unref (afp_name);
-  }
-
-  g_simple_async_result_complete (simple);
-}
-
-/*
- * g_vfs_afp_volume_map_id:
- * 
- * @volume: a #GVfsAfpVolume.
- * @map_function: a #AfpMapIDFunction.
- * @id: the id to be mapped to a name.
- * @cancellable: optional #GCancellable object, %NULL to ignore.
- * @callback: callback to call when the request is satisfied.
- * @user_data: the data to pass to callback function.
- * 
- * Asynchronously maps a user id, group id or uuid to a name.
- */
-void
-g_vfs_afp_volume_map_id (GVfsAfpVolume       *volume,
-                         AfpMapIDFunction     map_function,
-                         gint64               id,
-                         GCancellable        *cancellable,
-                         GAsyncReadyCallback  callback,
-                         gpointer             user_data)
-{
-  GVfsAfpVolumePrivate *priv;
-  GVfsAfpCommand *comm;
-  GSimpleAsyncResult *simple;
-  MapIDData *map_data;
-
-  g_return_if_fail (G_VFS_IS_AFP_VOLUME (volume));
-
-  priv = volume->priv;
-  
-  comm = g_vfs_afp_command_new (AFP_COMMAND_MAP_ID);
-
-  /* SubFunction*/
-  g_vfs_afp_command_put_byte (comm, map_function);
-
-  /* ID */
-  if (map_function == AFP_MAP_ID_FUNCTION_USER_ID_TO_NAME ||
-      map_function == AFP_MAP_ID_FUNCTION_GROUP_ID_TO_NAME)
-    g_vfs_afp_command_put_int32 (comm, id);
-  else
-    g_vfs_afp_command_put_int64 (comm, id);
-
-  simple = g_simple_async_result_new (G_OBJECT (volume), callback,
-                                      user_data, g_vfs_afp_volume_map_id);
-
-  map_data = g_slice_new0 (MapIDData);
-  map_data->function = map_function;
-  g_simple_async_result_set_op_res_gpointer (simple, map_data,
-                                             (GDestroyNotify)map_id_data_free);
-  
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
-                                     map_id_cb, cancellable, simple);
-  g_object_unref (comm);
-}
-
-/*
- * g_vfs_afp_volume_map_id_finish:
- * 
- * @volume: a #GVfsAfpVolume.
- * @result: a #GAsyncResult.
- * @map_function: (out) optional out parameter to get the #AfpMapIDFunction
- * which was used, %NULL to ignore.
- * @error: a #GError, %NULL to ignore.
- * 
- * Finalizes the asynchronous operation started by
- * g_vfs_afp_volume_map_id.
- * 
- * Returns: (transfer full): A string with the name of the id or %NULL
- * on error.
- */
-char *
-g_vfs_afp_volume_map_id_finish (GVfsAfpVolume   *volume,
-                                GAsyncResult     *res,
-                                AfpMapIDFunction *map_function,
-                                GError          **error)
-{
-  GSimpleAsyncResult *simple;
-  MapIDData *map_data;
-
-  g_return_val_if_fail (g_simple_async_result_is_valid (res, G_OBJECT (volume),
-                                                        g_vfs_afp_volume_map_id),
-                        NULL);
-
-  simple = (GSimpleAsyncResult *)res;
-
-  if (g_simple_async_result_propagate_error (simple, error))
-    return NULL;
-
-  map_data = g_simple_async_result_get_op_res_gpointer (simple);
-
-  if (map_function)
-    *map_function = map_data->function;
-  
-  return g_strdup (map_data->name);
-}
-
-static void
-get_filedir_parms_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
+get_filedir_parms_cb (GObject *source_object, GAsyncResult *result, gpointer user_data)
 {
   GVfsAfpConnection *conn = G_VFS_AFP_CONNECTION (source_object);
   GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (user_data);
@@ -1866,13 +1695,14 @@ get_filedir_parms_cb (GObject *source_object, GAsyncResult *res, gpointer user_d
   GVfsAfpReply *reply;
   GError *err = NULL;
   AfpResultCode res_code;
+  gboolean res;
 
   guint16 file_bitmap, dir_bitmap, bitmap;
   guint8 FileDir;
   gboolean directory;
   GFileInfo *info;
 
-  reply = g_vfs_afp_connection_send_command_finish (conn, res, &err);
+  reply = g_vfs_afp_connection_send_command_finish (conn, result, &err);
   if (!reply)
   {
     g_simple_async_result_take_error (simple, err);
@@ -1908,9 +1738,13 @@ get_filedir_parms_cb (GObject *source_object, GAsyncResult *res, gpointer user_d
   bitmap =  directory ? dir_bitmap : file_bitmap;
 
   info = g_file_info_new ();
-  g_vfs_afp_server_fill_info (volume->priv->server, info, reply, directory, bitmap);
-  
+  res = g_vfs_afp_server_fill_info (volume->priv->server, info, reply, directory, bitmap, &err);
   g_object_unref (reply);
+  if (!res)
+  {
+    g_simple_async_result_take_error (simple, err);
+    goto done;
+  }
 
   g_simple_async_result_set_op_res_gpointer (simple, info, g_object_unref);
 
@@ -1970,7 +1804,7 @@ g_vfs_afp_volume_get_filedir_parms (GVfsAfpVolume       *volume,
                                       g_vfs_afp_volume_get_filedir_parms);
                                       
 
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL,
                                      get_filedir_parms_cb, cancellable,
                                      simple);
   g_object_unref (comm);
@@ -2010,7 +1844,7 @@ g_vfs_afp_volume_get_filedir_parms_finish (GVfsAfpVolume  *volume,
 }
 
 static void
-get_fork_parms_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
+get_fork_parms_cb (GObject *source_object, GAsyncResult *result, gpointer user_data)
 {
   GVfsAfpConnection *conn = G_VFS_AFP_CONNECTION (source_object);
   GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (user_data);
@@ -2020,11 +1854,12 @@ get_fork_parms_cb (GObject *source_object, GAsyncResult *res, gpointer user_data
   GVfsAfpReply *reply;
   GError *err = NULL;
   AfpResultCode res_code;
+  gboolean res;
 
   guint16 file_bitmap;
   GFileInfo *info;
 
-  reply = g_vfs_afp_connection_send_command_finish (conn, res, &err);
+  reply = g_vfs_afp_connection_send_command_finish (conn, result, &err);
   if (!reply)
   {
     g_simple_async_result_take_error (simple, err);
@@ -2043,9 +1878,13 @@ get_fork_parms_cb (GObject *source_object, GAsyncResult *res, gpointer user_data
   g_vfs_afp_reply_read_uint16 (reply, &file_bitmap);
 
   info = g_file_info_new ();
-  g_vfs_afp_server_fill_info (priv->server, info, reply, FALSE, file_bitmap);
-
+  res = g_vfs_afp_server_fill_info (priv->server, info, reply, FALSE, file_bitmap, &err);
   g_object_unref (reply);
+  if (!res)
+  {
+    g_simple_async_result_take_error (simple, err);
+    goto done;
+  }
 
   g_simple_async_result_set_op_res_gpointer (simple, info, g_object_unref);
 
@@ -2095,7 +1934,7 @@ g_vfs_afp_volume_get_fork_parms (GVfsAfpVolume       *volume,
                                       g_vfs_afp_volume_get_fork_parms);
                                       
 
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL,
                                      get_fork_parms_cb, cancellable,
                                      simple);
   g_object_unref (comm);
@@ -2223,7 +2062,7 @@ g_vfs_afp_volume_set_fork_size (GVfsAfpVolume       *volume,
   simple = g_simple_async_result_new (G_OBJECT (volume), callback, user_data,
                                       g_vfs_afp_volume_set_fork_size);
   
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL,
                                      set_fork_parms_cb, cancellable, simple);
   g_object_unref (comm);
 }
@@ -2369,7 +2208,7 @@ g_vfs_afp_volume_set_unix_privs (GVfsAfpVolume       *volume,
   simple = g_simple_async_result_new (G_OBJECT (volume), callback,
                                       user_data, g_vfs_afp_volume_set_unix_privs);
 
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL,
                                      set_unix_privs_cb, cancellable, simple);
   g_object_unref (comm);
 }
@@ -2491,7 +2330,13 @@ enumerate_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
     bitmap =  directory ? dir_bitmap : file_bitmap;
     
     info = g_file_info_new ();
-    g_vfs_afp_server_fill_info (priv->server, info, reply, directory, bitmap);
+    if (!g_vfs_afp_server_fill_info (priv->server, info, reply, directory, bitmap, &err))
+    {
+      g_object_unref (reply);
+      g_simple_async_result_take_error (simple, err);
+      goto done;
+    }
+    
     g_ptr_array_add (infos, info);
 
     g_vfs_afp_reply_seek (reply, start_pos + struct_length, G_SEEK_SET);
@@ -2528,6 +2373,8 @@ g_vfs_afp_volume_enumerate (GVfsAfpVolume       *volume,
                             gpointer             user_data)
 {
   GVfsAfpVolumePrivate *priv;
+
+  const GVfsAfpServerInfo *info;
   gint32 max;
   
   GVfsAfpCommand *comm;
@@ -2539,8 +2386,10 @@ g_vfs_afp_volume_enumerate (GVfsAfpVolume       *volume,
 
   simple = g_simple_async_result_new (G_OBJECT (volume), callback,
                                       user_data, g_vfs_afp_volume_enumerate);
+
+  info = g_vfs_afp_server_get_info (priv->server);
   
-  max = (priv->server->version >= AFP_VERSION_3_1) ? G_MAXINT32 : G_MAXINT16;
+  max = (info->version >= AFP_VERSION_3_1) ? G_MAXINT32 : G_MAXINT16;
   /* Can't enumerate any more files */
   if (start_index > max)
   {
@@ -2549,7 +2398,7 @@ g_vfs_afp_volume_enumerate (GVfsAfpVolume       *volume,
     return;
   }
   
-  if (priv->server->version >= AFP_VERSION_3_1)
+  if (info->version >= AFP_VERSION_3_1)
     comm = g_vfs_afp_command_new (AFP_COMMAND_ENUMERATE_EXT2);
   else
     comm = g_vfs_afp_command_new (AFP_COMMAND_ENUMERATE_EXT);
@@ -2573,7 +2422,7 @@ g_vfs_afp_volume_enumerate (GVfsAfpVolume       *volume,
 
   
   /* StartIndex and MaxReplySize */
-  if (priv->server->version >= AFP_VERSION_3_1)
+  if (info->version >= AFP_VERSION_3_1)
   {
     g_vfs_afp_command_put_int32 (comm, start_index);
     g_vfs_afp_command_put_int32 (comm, ENUMERATE_EXT2_MAX_REPLY_SIZE);
@@ -2587,7 +2436,7 @@ g_vfs_afp_volume_enumerate (GVfsAfpVolume       *volume,
   /* Pathname */
   g_vfs_afp_command_put_pathname (comm, directory);
   
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL,
                                      enumerate_cb, cancellable, simple);
   g_object_unref (comm);
 }
@@ -2727,7 +2576,7 @@ g_vfs_afp_volume_exchange_files (GVfsAfpVolume       *volume,
   simple = g_simple_async_result_new (G_OBJECT (volume), callback, user_data,
                                       g_vfs_afp_volume_exchange_files);
   
-  g_vfs_afp_connection_send_command (priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (priv->conn, comm, NULL,
                                      close_replace_exchange_files_cb,
                                      cancellable, simple);
   g_object_unref (comm); 
@@ -2783,7 +2632,7 @@ write_ext_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
   }
 
   res_code = g_vfs_afp_reply_get_result_code (reply);
-  if (!(res_code == AFP_RESULT_NO_ERROR || res_code == AFP_RESULT_LOCK_ERR))
+  if (res_code != AFP_RESULT_NO_ERROR)
   {
     g_object_unref (reply);
 
@@ -2797,6 +2646,10 @@ write_ext_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
         g_simple_async_result_set_error (simple, G_IO_ERROR, G_IO_ERROR_NO_SPACE,
                                   _("Not enough space on volume"));
         break;
+      case AFP_RESULT_LOCK_ERR:
+        g_simple_async_result_set_error (simple, G_IO_ERROR, G_IO_ERROR_FAILED,
+                                         _("File is locked by another user"));
+        break;
       default:
         g_simple_async_result_take_error (simple, afp_result_code_to_gerror (res_code));
         break;
@@ -2842,7 +2695,7 @@ g_vfs_afp_volume_write_to_fork (GVfsAfpVolume       *volume,
                                 gpointer             user_data)
 {
   GVfsAfpCommand *comm;
-  guint32 req_count;
+  guint32 max_req_count, req_count;
   GSimpleAsyncResult *simple;
 
   g_return_if_fail (G_VFS_IS_AFP_VOLUME (volume));
@@ -2855,16 +2708,18 @@ g_vfs_afp_volume_write_to_fork (GVfsAfpVolume       *volume,
   g_vfs_afp_command_put_int16 (comm, fork_refnum);
   /* Offset */
   g_vfs_afp_command_put_int64 (comm, offset);
+  
   /* ReqCount */
+  max_req_count = g_vfs_afp_server_get_max_request_size (volume->priv->server) - 20;
   req_count = MIN (buffer_size, G_MAXUINT32);
   g_vfs_afp_command_put_int64 (comm, req_count);
 
-  g_vfs_afp_command_set_buffer (comm, buffer, buffer_size);
+  g_vfs_afp_command_set_buffer (comm, buffer, req_count);
 
   simple = g_simple_async_result_new (G_OBJECT (volume), callback, user_data,
                                       g_vfs_afp_volume_write_to_fork);
   
-  g_vfs_afp_connection_send_command (volume->priv->server->conn, comm, NULL,
+  g_vfs_afp_connection_send_command (volume->priv->conn, comm, NULL,
                                      write_ext_cb, cancellable, simple);
   g_object_unref (comm);
 }
@@ -2994,7 +2849,7 @@ g_vfs_afp_volume_read_from_fork (GVfsAfpVolume       *volume,
   simple = g_simple_async_result_new (G_OBJECT (volume), callback, user_data,
                                       g_vfs_afp_volume_read_from_fork);
   
-  g_vfs_afp_connection_send_command (volume->priv->server->conn, comm, buffer,
+  g_vfs_afp_connection_send_command (volume->priv->conn, comm, buffer,
                                      read_ext_cb, cancellable, simple);
   g_object_unref (comm);
 }
