@@ -33,6 +33,7 @@
 #endif
 
 #include "gvfsjobmount.h"
+#include "gvfsjobunmount.h"
 #include "gvfsjobenumerate.h"
 #include "gvfsjobqueryinfo.h"
 #include "gvfsjobqueryfsinfo.h"
@@ -53,6 +54,7 @@
 #include "gvfsjobcopy.h"
 
 #include "gvfsafpserver.h"
+#include "gvfsafpvolume.h"
 
 #include "gvfsbackendafp.h"
 
@@ -1019,20 +1021,12 @@ close_write_get_fork_parms_cb (GObject *source_object, GAsyncResult *res, gpoint
 
   AfpHandle *afp_handle = (AfpHandle *)job->handle;
 
-  GError *err = NULL;
   GFileInfo *info;
 
-  info = g_vfs_afp_volume_get_fork_parms_finish (volume, res, &err);
-  if (!info)
-  {
-    g_vfs_job_failed_from_error (G_VFS_JOB (job), err);
-    g_error_free (err);
+  info = g_vfs_afp_volume_get_fork_parms_finish (volume, res, NULL);
+  if (info)
+    g_vfs_job_close_write_set_etag (job, g_file_info_get_etag (info));
 
-    afp_handle_free (afp_handle);
-    return;
-  }
-
-  g_vfs_job_close_write_set_etag (job, g_file_info_get_etag (info));
   close_fork (volume, G_VFS_JOB (job), afp_handle);
 }
   
@@ -1820,27 +1814,27 @@ try_query_fs_info (GVfsBackend *backend,
 static void
 get_name_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
-  GVfsAfpVolume *volume = G_VFS_AFP_VOLUME (source_object);
+  GVfsAfpServer *server = G_VFS_AFP_SERVER (source_object);
   GVfsJobQueryInfo *job = G_VFS_JOB_QUERY_INFO (user_data);
 
   char *name;
-  AfpMapIDFunction map_function;
+  GVfsAfpMapIDFunction map_function;
   guint outstanding_requests;
 
-  name = g_vfs_afp_volume_map_id_finish (volume, res, &map_function, NULL);
+  name = g_vfs_afp_server_map_id_finish (server, res, &map_function, NULL);
   if (name)
   {
     switch (map_function)
     {
-      case AFP_MAP_ID_FUNCTION_USER_ID_TO_NAME:
+      case GVFS_AFP_MAP_ID_FUNCTION_USER_ID_TO_NAME:
         g_file_info_set_attribute_string (job->file_info, G_FILE_ATTRIBUTE_OWNER_USER,
                                           name);
         break;
-      case AFP_MAP_ID_FUNCTION_USER_ID_TO_UTF8_NAME:
+      case GVFS_AFP_MAP_ID_FUNCTION_USER_ID_TO_UTF8_NAME:
         g_file_info_set_attribute_string (job->file_info, G_FILE_ATTRIBUTE_OWNER_USER_REAL,
                                           name);
         break;
-      case AFP_MAP_ID_FUNCTION_GROUP_ID_TO_NAME:
+      case GVFS_AFP_MAP_ID_FUNCTION_GROUP_ID_TO_NAME:
         g_file_info_set_attribute_string (job->file_info, G_FILE_ATTRIBUTE_OWNER_GROUP,
                                           name);
         break;
@@ -1906,16 +1900,16 @@ query_info_get_filedir_parms_cb (GObject *source_object, GAsyncResult *res, gpoi
 
     if (g_file_attribute_matcher_matches (matcher, G_FILE_ATTRIBUTE_OWNER_USER))
     {
-      g_vfs_afp_volume_map_id (volume,
-                               AFP_MAP_ID_FUNCTION_USER_ID_TO_NAME, uid,
+      g_vfs_afp_server_map_id (afp_backend->server,
+                               GVFS_AFP_MAP_ID_FUNCTION_USER_ID_TO_NAME, uid,
                                G_VFS_JOB (job)->cancellable, get_name_cb, job);
       outstanding_requests++;
     }
     
     if (g_file_attribute_matcher_matches (matcher, G_FILE_ATTRIBUTE_OWNER_USER_REAL))
     {
-      g_vfs_afp_volume_map_id (volume,
-                               AFP_MAP_ID_FUNCTION_USER_ID_TO_UTF8_NAME, uid,
+      g_vfs_afp_server_map_id (afp_backend->server,
+                               GVFS_AFP_MAP_ID_FUNCTION_USER_ID_TO_UTF8_NAME, uid,
                                G_VFS_JOB (job)->cancellable, get_name_cb, job);
       outstanding_requests++;
     }
@@ -1928,8 +1922,8 @@ query_info_get_filedir_parms_cb (GObject *source_object, GAsyncResult *res, gpoi
 
     gid = g_file_info_get_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_GID);
 
-    g_vfs_afp_volume_map_id (volume,
-                             AFP_MAP_ID_FUNCTION_GROUP_ID_TO_NAME, gid,
+    g_vfs_afp_server_map_id (afp_backend->server,
+                             GVFS_AFP_MAP_ID_FUNCTION_GROUP_ID_TO_NAME, gid,
                              G_VFS_JOB (job)->cancellable, get_name_cb, job);
     outstanding_requests++;
   }
@@ -1994,6 +1988,23 @@ try_query_info (GVfsBackend *backend,
   return TRUE;
 }
 
+static void
+do_unmount (GVfsBackend *backend,
+            GVfsJobUnmount *job,
+            GMountUnmountFlags flags,
+            GMountSource *mount_source)
+{
+  GVfsBackendAfp *afp_backend = G_VFS_BACKEND_AFP (backend);
+
+  if (!(flags & G_MOUNT_UNMOUNT_FORCE))
+  {
+    g_vfs_afp_server_logout_sync (afp_backend->server, G_VFS_JOB (job)->cancellable,
+                                  NULL);
+  }
+
+  g_vfs_job_succeeded (G_VFS_JOB (job));
+}
+
 static void
 do_mount (GVfsBackend *backend,
           GVfsJobMount *job,
@@ -2005,7 +2016,8 @@ do_mount (GVfsBackend *backend,
 
   gboolean res;
   GError *err = NULL;
-  
+
+  const GVfsAfpServerInfo *info;
   GMountSpec *afp_mount_spec;
   char       *server_name;
   char       *display_name;
@@ -2034,10 +2046,12 @@ do_mount (GVfsBackend *backend,
   g_vfs_backend_set_mount_spec (backend, afp_mount_spec);
   g_mount_spec_unref (afp_mount_spec);
 
-  if (afp_backend->server->utf8_server_name)
-    server_name = afp_backend->server->utf8_server_name;
+  info = g_vfs_afp_server_get_info (afp_backend->server);
+  
+  if (info->utf8_server_name)
+    server_name = info->utf8_server_name;
   else
-    server_name = afp_backend->server->server_name;
+    server_name = info->server_name;
   
   if (afp_backend->user)
     /* Translators: first %s is volumename, second username and third servername */ 
@@ -2145,6 +2159,7 @@ g_vfs_backend_afp_class_init (GVfsBackendAfpClass *klass)
 
   backend_class->try_mount = try_mount;
   backend_class->mount = do_mount;
+  backend_class->unmount = do_unmount;
   backend_class->try_query_info = try_query_info;
   backend_class->try_query_fs_info = try_query_fs_info;
   backend_class->try_set_attribute = try_set_attribute;
