@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Author: David Zeuthen <davidz@redhat.com>
  */
@@ -566,14 +566,10 @@ gvfs_udisks2_utils_is_drive_on_our_seat (UDisksDrive *drive)
    * seat.
    *
    * Note that seat support was added in udisks 1.95.0 (and so was the
-   * UDISKS_CHECK_VERSION macro) - for now, be compatible with older
-   * versions instead of bumping requirement in configure.ac
+   * UDISKS_CHECK_VERSION macro).
    */
-#ifdef UDISKS_CHECK_VERSION
-# if UDISKS_CHECK_VERSION(1,95,0)
   drive_seat = udisks_drive_get_seat (drive);
-# endif
-#endif
+
   if (drive_seat == NULL || strlen (drive_seat) == 0)
     {
       ret = TRUE;
